using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.Drawing.Imaging;


namespace NavsGo
{
    public class ImageButton 
    {
        internal Rectangle clientArea;
        internal Rectangle imageArea;
        internal Rectangle imageDownArea;
        internal Form owner;
        internal Bitmap image;
        internal bool pushed = false;
        internal Point location;
        internal Point start;

        private int width;
        private int height;
        
        internal bool pushedOneTime = false;
        
        public object Tag { get; set; }

        public bool IsPressedOneTime
        {
            get { return pushedOneTime; }
            set { pushedOneTime = value; }
        }
     
    

        public ImageButton(Form owner,Point location)
        {
            this.owner = owner;
            Attach(owner);
            this.location = location;
            this.clientArea = new Rectangle(location.X, location.Y, 46, 46);
        }

      

        public Bitmap Image
        {
            get { return image;}
            set 
            { 
                image = value;
                if (image != null)
                {
                    width = image.Width;
                    height = image.Height;
                    imageArea = new Rectangle(location.X + (clientArea.Width - image.Width) / 2 - 1, location.Y + (clientArea.Height - image.Height) / 2 - 1, image.Width, image.Height);
                    imageDownArea = new Rectangle(imageArea.X + 2, imageArea.Y + 2, imageArea.Width + 2, imageArea.Height + 2);
                }
            }
        }

        public bool HitTest(int x, int y)
        {
            return clientArea.Contains(x, y);
        }

        private void Attach(Form owner)
        {           
            owner.MouseDown += new MouseEventHandler(owner_MouseDown);         
            owner.MouseUp += new MouseEventHandler(owner_MouseUp);
        }

        public virtual void owner_MouseUp(object sender, MouseEventArgs e)
        {
            if (pushed)
            {
                using (Graphics gx = owner.CreateGraphics())
                {
                    this.pushed = false;
                    this.Paint(gx);
                }
                pushed = false;
            }

//            SendMessage(ButtonCode, "Button Pressed");
        }

        public virtual void owner_MouseDown(object sender, MouseEventArgs e)
        {
            if (this.HitTest(e.X, e.Y))
            {
                using (Graphics gx = owner.CreateGraphics())
                {
                    this.pushed = true;
                    IsPressedOneTime = true;
                    this.Paint(gx);
                }
                start = new Point(e.X, e.Y);            
            }
        }
    
        public void Paint(Graphics gx)
        {
            //Paint the Rectangle
            gx.DrawGradientRoundedRectangle(clientArea, Color.FromArgb(132, 178, 222),
                         Color.FromArgb(0, 101, 189), Color.FromArgb(0, 101, 189), new Size(8, 8)); 

            //Paint the picture
            ImageAttributes attrib = new ImageAttributes();
            Color color = GetTransparentColor(image);
            attrib.SetColorKey(color, color);

            if (!pushed)
            {
                gx.DrawImage(image, imageArea, 0, 0, imageArea.Width, imageArea.Height, GraphicsUnit.Pixel, attrib);
            }
            else
            {
                gx.DrawImage(image, imageDownArea, 0, 0, imageDownArea.Width, imageDownArea.Height, GraphicsUnit.Pixel, attrib);
            }
        }

        internal Color GetTransparentColor(Bitmap image)
        {
            return image.GetPixel(0, 0);
        }
    
    }
}
